from verbecc.src.conjugator.conjugator import Conjugator
from verbecc.src.defs.types.alternates_behavior import AlternatesBehavior
from verbecc.src.defs.types.data_types import (
    MoodsConjugation,
    MoodConjugation,
    TenseConjugation,
    PersonConjugation,
    VerbInfo,
    Conjugation,
)
from verbecc.src.defs.constants.grammar_defines import SUPPORTED_LANGUAGES
from verbecc.src.parsers.mood import Mood
from verbecc.src.parsers.verb import Verb
from verbecc.src.defs.types.exceptions import (
    ConjugatorError,
    ConjugationTemplateError,
    ConjugationsParserError,
    VerbNotFoundError,
    TemplateNotFoundError,
    InvalidLangError,
    InvalidMoodError,
    InvalidTenseError,
)



from typing import Dict, List, Tuple

from verbecc.src.inflectors.inflector import Inflector
from verbecc.src.conjugator.conjugation_object import ConjugationObjects


class InflectorRo(Inflector):
    @property
    def lang(self) -> str:
        return "ro"

    def __init__(self):
        super(InflectorRo, self).__init__()

    def _add_subjunctive_relative_pronoun(self, s, tense_name):
        tokens = s.split(" ")
        if tense_name == "prezent":
            tokens.insert(1, "să")
        elif tense_name == "perfect":
            tokens.insert(1, "să fi")
        return " ".join(tokens)

    def _add_adverb_if_applicable(self, s, mood_name, tense_name):
        if mood_name == "imperativ" and tense_name == "negativ":
            return "nu " + s
        return s

    """TODO: There are two types of reflexive verbs in Romanian: 
    preceded by the reflexive pronouns “se” (in the accusative) and “și” (in the dative).
    """

    def _get_default_pronoun(self, person, gender="m", is_reflexive=False):
        ret = ""
        if person == "1s":
            ret = "eu"
            if is_reflexive:
                ret += " mă"
        elif person == "2s":
            ret = "tu"
            if is_reflexive:
                ret += " te"
        elif person == "3s":
            ret = "el"
            if gender == "f":
                ret = "ea"
            if is_reflexive:
                ret += " se"
        elif person == "1p":
            ret = "noi"
            if is_reflexive:
                ret += " ne"
        elif person == "2p":
            ret = "voi"
            if is_reflexive:
                ret += " vă"
        elif person == "3p":
            ret = "ei"
            if gender == "f":
                ret = "ele"
            if is_reflexive:
                ret += " se"
        return ret

    def _get_tenses_conjugated_without_pronouns(self):
        return ["participiu", "afirmativ", "imperativ", "negativ", "gerunziu"]

    def _get_auxilary_verb(self, co, mood_name, tense_name):
        if tense_name in ("viitor-1", "viitor-2"):
            return "voi"
        elif tense_name == "viitor-1-popular":
            return co.verb.infinitive
        return "avea"

    def _get_infinitive_mood_name(self):
        return "infinitiv"

    def _get_indicative_mood_name(self):
        return "indicativ"

    def _get_subjunctive_mood_name(self):
        return "conjunctiv"

    def _get_conditional_mood_name(self):
        return "conditional"

    def _get_participle_mood_name(self):
        return "participiu"

    def _get_participle_tense_name(self):
        return "participiu"

    def _get_compound_conjugations_aux_verb_map(
        self,
    ) -> Dict[str, Dict[str, Tuple[str, ...]]]:
        # TODO: those last three don't actually use an auxiliary verb, refactor to make aux verb optional
        return {
            "indicativ": {
                "perfect-compus": ("indicativ", "prezent"),
                "viitor-1": ("indicativ", "prezent"),
                "viitor-2": ("indicativ", "prezent"),
                "viitor-1-popular": ("conjunctiv", "prezent"),
                "viitor-2-popular": ("indicativ", "prezent"),
            },
            "conjunctiv": {"perfect": ("indicativ", "prezent")},
            "conditional": {
                "prezent": ("indicativ", "prezent"),
                "perfect": ("indicativ", "prezent"),
            },
        }

    def _auxilary_verb_uses_alternate_conjugation(self, tense_name) -> bool:
        return tense_name.startswith("viitor")

    def _compound_primary_verb_conjugation_uses_infinitive(
        self, mood_name: str, tense_name: str
    ):
        if mood_name == "indicativ" and tense_name == "viitor-1":
            return True
        elif mood_name == "conditional" and tense_name == "prezent":
            return True
        return False

    def _modify_aux_verb_conj_if_applicable(
        self, aux_conj: List[str], mood_name: str, tense_name: str
    ):
        """E.g. for Romanian conditional present 'eu aş avea, tu ai avea, el ar avea, ...'
        and also the Romanian conditional present e.g. 'eu	aş fi avut, tu ai fi avut, ...'
        although the ' fi' is added by _add_compound_aux_verb_suffix_if_applicable

        Normally Romanian aux_conj would be the indicativ prezent tense of avea i.e.
            ["eu am", "tu ai", "el a", "noi am", "voi aţi", "ei au"]
        but for conditional it's supposed to be
            ["eu aş", "tu ai", "el ar", "noi am", "voi aţi", "ei ar"]
        """
        if mood_name == "conditional" and tense_name in ("prezent", "perfect"):
            sub_aux_conj = ["aş", "ai", "ar", "am", "aţi", "ar"]
            for i, c in enumerate(aux_conj):
                pronoun, _ = c.split(" ")
                aux_conj[i] = f"{pronoun} {sub_aux_conj[i]}"
        return aux_conj

    def _add_compound_aux_verb_suffix_if_applicable(
        self, s: str, mood_name: str, tense_name: str
    ) -> str:
        """
        E.g. for Romanian indicativ viitor-ii this appends " fi" to make "eu am să fi avut" etc.
        """
        if (mood_name == "indicativ" and tense_name == "viitor-2") or (
            mood_name == "conditional" and tense_name == "perfect"
        ):
            return s + " fi"
        elif mood_name == "indicativ" and tense_name == "viitor-2-popular":
            return s + " să fi"
        # TODO: Research. Some sources e.g. verbix.com don't include " să"
        # elif mood_name == "indicativ" and tense_name == "viitor-1":
        #    return s + " să"
        return s

    def _insert_compound_aux_verb_prefix_if_applicable(
        self, s: str, mood_name: str, tense_name: str
    ) -> str:
        """
        Used by Romanian viitor-1-popular
        "eu o să fac, tu o să faci, ..."
        """
        if mood_name == "indicativ" and tense_name == "viitor-1-popular":
            tokens = s.split()
            return tokens[0] + " o să " + tokens[1]
        return s

    def _compound_has_no_primary_verb(self, mood_name: str, tense_name: str) -> bool:
        """Used for Romanian viitor-1-popular"""
        if mood_name == "indicativ" and tense_name == "viitor-1-popular":
            return True
        return False

    def _compound_has_no_aux_verb(self, mood_name: str, tense_name: str) -> bool:
        """Used for Romanian conjunctiv perfect"""
        if mood_name == "conjunctiv" and tense_name == "perfect":
            return True
        return False



from typing import Union, Dict, List

# PersonConjugation format depends on AlternatesBehavior
# With AlternatesBehavior.All, its a List[str]
# With AlternatesBehvioar.FirstOnly or SecondOnly, it's a str
PersonConjugation = Union[List[str], str]
TenseConjugation = List[PersonConjugation]
MoodConjugation = Dict[str, TenseConjugation]
MoodsConjugation = Dict[str, MoodConjugation]  # the "moods" section
VerbInfo = Dict[str, Union[str, bool, float]]  # the "verb" section
Conjugation = Dict[str, Union[VerbInfo, MoodsConjugation]]



MOOD_MAP = {
    "indicative": {
        "ca": "indicatiu",
        "es": "indicativo",
        "fr": "indicatif",
        "it": "indicativo",
        "pt": "indicativo",
        "ro": "indicativ",
    },
    "subjunctive": {
        "ca": "subjuntiu",
        "es": "subjuntivo",
        "fr": "subjonctif",
        "it": "congiuntivo",
        "pt": "subjuntivo",
        "ro": "conjunctiv",
    },
    "imperative": {
        "ca": "imperatiu",
        "es": "imperativo",
        "fr": "imperatif",
        "it": "imperativo",
        "pt": "imperativo",
        "ro": "imperativ",
    },
    "conditional": {
        "ca": "condicional",
        "es": "condicional",
        "fr": "conditionnel",
        "it": "condizionale",
        "pt": "condicional",
        "ro": "<not implemented>",  # The conditional mood is formed by combining the conditional particle "ar" with the infinitive form of the verb
    },
    "infinitive": {
        "ca": "infinitiu",
        "es": "infinitivo",
        "fr": "infinitif",
        "it": "infinito",
        "pt": "infinitivo",
        "ro": "infinitiv",
    },
    # TODO: Make French consistent with other languages i.e.
    # make gerund and past-participle different moods i.e.
    # participe.participe-présent -> participe-présent.participe
    # participe.participe-passé -> participe-passé.participe
    "gerund": {
        "ca": "gerundi",  # gerundi.gerundi
        "es": "gerundio",  # gerundio.gerundio
        "fr": "participe",  # participe.participe-présent is gerund in French
        "it": "infinito",  # infinito.gerundio is gerund in Italian
        "pt": "gerúndio",  # gerúndio.gerúndio
        "ro": "gerunziu",  # gerunziu.gerunziu
    },
    "participle": {
        "ca": "particip",  # particip.particip
        "es": "participo",  # participo.participo
        "fr": "participe",  # participe.participe-présent, participe.participe-passé
        "it": "participio",  # participio.participio
        "pt": "particípio",  # particípio.particípio
        "ro": "participiu",  # participiu.participiu
    },
}

TENSE_MAP = {
    "present": {
        "ca": "present",
        "es": "presente",
        "fr": "présent",
        "it": "presente",
        "pt": "presente",
        "ro": "prezent",
    },
    "imperfect": {
        "ca": "imperfet",
        "es": "pretérito-imperfecto",
        "fr": "imparfait",
        "it": "imperfetto",
        "pt": "pretérito-imperfeito",
    },
    "simple-past": {
        "ca": "passat-simple",
        "es": "pretérito-perfecto-simple",
        "fr": "passé-simple",
        "it": "passato-remoto",  # The passato prossimo (compound tense) is roughly equivalent to the present perfect and past simple tenses
        # however passato-remoto is the literal simple past tense e.g. 'io fui, tu fosti, lui fu, noi fummo',
        "pt": "pretérito-perfeito",
        "ro": "perfect-simplu",
    },
    "future": {
        "ca": "futur",
        "es": "futuro",
        "fr": "futur-simple",
        "it": "futuro",
        "pt": "futuro-do-presente",
        "ro": "viitor-1",  # Romanian has viitor-1, viitor-2, viitor-1-popular, viitor-2-popular
    },
    "gerund": {
        "ca": "gerundi",
        "es": "gerundio",
        "fr": "participe-présent",  # TODO: Make French consistent
        "it": "gerundio",
        "pt": "gerúndio",
        "ro": "gerunziu",
    },
    "past-participle": {
        "ca": "particip",
        "es": "participo",
        "fr": "participe-passé",
        "it": "participio",
        "pt": "particípio",
        "ro": "participiu",
    },
}


def xmood(lang, m):
    """Takes a mood name in English and translates it to the specified language

    :raises: Exception if mood or lang doesn't exist
    """
    return MOOD_MAP[m][lang]


def xtense(lang, t):
    """Takes a tense name in English and translates it to the specified language

    :raises: Exception if tense or lang doesn't exist
    """
    return TENSE_MAP[t][lang]



from lxml import etree

from verbecc.src.parsers.person_ending import PersonEnding
from verbecc.src.parsers.tense_template import TenseTemplate


def test_tense_and_person():
    tense_elem = etree.fromstring(
        """<present>
        <p><i>ie</i><i>ye</i></p>
        <p><i>ies</i><i>yes</i></p>
        <p><i>ie</i><i>ye</i></p>
        <p><i>yons</i></p>
        <p><i>yez</i></p>
        <p><i>ient</i><i>yent</i></p>
        </present>""",
        None,
    )
    tense_name = "present"
    tense = TenseTemplate(tense_elem)
    assert tense.name == tense_name
    assert tense.person_endings[0].get_ending() == "ie"
    assert tense.person_endings[0].get_alternate_ending_if_available() == "ye"
    assert tense.person_endings[0].get_person() == "1s"
    assert tense.person_endings[3].get_ending() == "yons"
    assert tense.person_endings[3].get_alternate_ending_if_available() == "yons"
    assert tense.person_endings[3].get_person() == "1p"



from lxml import etree
from typing import Dict

from verbecc.src.parsers.tense_template import TenseTemplate


class Mood:
    def __init__(self, mood_elem: etree._Element):
        self.name = mood_elem.tag.lower()
        self.tenses: Dict[str, TenseTemplate] = {}
        for tense_elem in mood_elem:  # type: ignore
            self.tenses[tense_elem.tag] = TenseTemplate(tense_elem)

    def __repr__(self):
        return "name={} tenses={}".format(self.name, self.tenses)



from typing import Dict, List, Tuple

from verbecc.src.inflectors.inflector import Inflector
from verbecc.src.utils import string_utils
from verbecc.src.conjugator.conjugation_object import ConjugationObjects

"""
DR & MRS VANDERTRAMPP verbs
"""
VERBS_CONJUGATED_WITH_ETRE = [
    "aller",
    "arriver",
    "descendre",
    "redescendre",
    "entrer",
    "rentrer",
    "monter",
    "remonter",
    "mourir",
    "naître",
    "renaître",
    "partir",
    "repartir",
    "passer",
    "rester",
    "retourner",
    "sortir",
    "ressortir",
    "tomber",
    "retomber",
    "venir",
    "devenir",
    "parvenir",
    "revenir",
]

VERBS_THAT_CANNOT_BE_REFLEXIVE_OTHER_THAN_IMPERSONAL_VERBS = ["être", "aller", "avoir"]


class InflectorFr(Inflector):
    @property
    def lang(self) -> str:
        return "fr"

    def __init__(self):
        super(InflectorFr, self).__init__()

    def get_verbs_that_start_with(self, query: str, max_results: int) -> List[str]:
        query = query.lower()
        is_reflexive, query = self._split_reflexive(query)
        matches = self._verb_parser.get_verbs_that_start_with(query, max_results)
        if is_reflexive:
            matches = [
                self._add_reflexive_pronoun(m)
                for m in matches
                if self._verb_can_be_reflexive(m)
            ]
        return matches

    def _is_impersonal_verb(self, infinitive: str) -> bool:
        ret = False
        verb = self.find_verb_by_infinitive(infinitive)
        template = self.find_template(verb.template)
        if len(template.moods["indicatif"].tenses["présent"].person_endings) < 6:
            ret = True
        return ret

    def _verb_can_be_reflexive(self, infinitive: str) -> bool:
        return (
            not self._is_impersonal_verb(infinitive)
            and infinitive
            not in VERBS_THAT_CANNOT_BE_REFLEXIVE_OTHER_THAN_IMPERSONAL_VERBS
        )

    def _split_reflexive(self, infinitive: str) -> Tuple[bool, str]:
        is_reflexive = False
        if infinitive.startswith("se "):
            is_reflexive = True
            infinitive = infinitive[3:]
        elif infinitive.startswith("s'"):
            is_reflexive = True
            infinitive = infinitive[2:]
        return is_reflexive, infinitive

    def _add_reflexive_pronoun(self, s: str) -> str:
        if string_utils.starts_with_vowel(s, h_is_vowel=True):
            return "s'" + s
        else:
            return "se " + s

    def _add_subjunctive_relative_pronoun(self, s: str, tense_name: str) -> str:
        if string_utils.starts_with_vowel(s, h_is_vowel=True):
            return "qu'" + s
        else:
            return "que " + s

    def _get_pronoun_suffix(self, person: str, gender: str = "m") -> str:
        return "-" + self._get_default_pronoun(person, gender).replace("tu", "toi")

    def _get_default_pronoun(
        self, person: str, gender: str = "m", is_reflexive: bool = False
    ) -> str:
        ret = ""
        if person == "1s":
            ret = "je"
            if is_reflexive:
                ret += " me"
        elif person == "2s":
            ret = "tu"
            if is_reflexive:
                ret += " te"
        elif person == "3s":
            ret = "il"
            if gender == "f":
                ret = "elle"
            if is_reflexive:
                ret += " se"
        elif person == "1p":
            ret = "nous"
            if is_reflexive:
                ret += " nous"
        elif person == "2p":
            ret = "vous"
            if is_reflexive:
                ret += " vous"
        elif person == "3p":
            ret = "ils"
            if gender == "f":
                ret = "elles"
            if is_reflexive:
                ret += " se"
        return ret

    def _get_tenses_conjugated_without_pronouns(self) -> List[str]:
        return [
            "infinitif-présent",
            "participe-présent",
            "imperatif-présent",
            "participe-passé",
        ]

    def _get_auxilary_verb(
        self,
        co: ConjugationObjects,
        mood_name: str,
        tense_name: str,
    ) -> str:
        ret = "avoir"
        if co.verb.infinitive in VERBS_CONJUGATED_WITH_ETRE or co.is_reflexive:
            ret = "être"
        return ret

    def _is_auxilary_verb_inflected(self, auxilary_verb: str) -> bool:
        return auxilary_verb == "être"

    def _get_infinitive_mood_name(self) -> str:
        return "infinitif"

    def _get_indicative_mood_name(self) -> str:
        return "indicatif"

    def _get_subjunctive_mood_name(self) -> str:
        return "subjonctif"

    def _get_conditional_mood_name(self) -> str:
        return "conditionnel"

    def _get_participle_mood_name(self) -> str:
        return "participe"

    def _get_participle_tense_name(self) -> str:
        return "participe-passé"

    def _combine_pronoun_and_conj(self, pronoun: str, conj: str) -> str:
        ret = ""
        if pronoun[-1] == "e" and string_utils.starts_with_vowel(conj, h_is_vowel=True):
            ret += pronoun[:-1] + "'"
        else:
            ret += pronoun + " "
        ret += conj
        return ret

    def _add_present_participle_if_applicable(
        self, s: str, is_reflexive: bool, tense_name: str
    ) -> str:
        ret = s
        if is_reflexive and tense_name == self._get_participle_tense_name():
            ret += "étant "
        return ret

    def _add_reflexive_pronoun_or_pronoun_suffix_if_applicable(
        self, s: str, is_reflexive: bool, mood_name: str, tense_name: str, person: str
    ) -> str:
        if is_reflexive:
            if mood_name != "imperatif":
                s = self._add_reflexive_pronoun(s)
            else:
                s += self._get_pronoun_suffix(person)
        return s

    def _compound_conjugation_not_applicable(
        self, is_reflexive, mood_name, hv_tense_name
    ) -> bool:
        return (
            is_reflexive
            and mood_name == "imperatif"
            and hv_tense_name == "imperatif-présent"
        )

    def _get_compound_conjugations_aux_verb_map(
        self,
    ) -> Dict[str, Dict[str, Tuple[str, ...]]]:
        """
        compound conjugations are formed using an auxiliary
        verb (aka helping verb)
        this method returns a Dictionary mapping of
        [compound-mood][compound-tense] to (aux-verb-mood, aux-verb-tense)
        """
        return {
            "indicatif": {
                "passé-composé": ("indicatif", "présent"),
                "plus-que-parfait": ("indicatif", "imparfait"),
                "futur-antérieur": ("indicatif", "futur-simple"),
                "passé-antérieur": ("indicatif", "passé-simple"),
            },
            "subjonctif": {
                "passé": ("subjonctif", "présent"),
                "plus-que-parfait": ("subjonctif", "imparfait"),
            },
            "conditionnel": {"passé": ("conditionnel", "présent")},
            "imperatif": {"imperatif-passé": ("imperatif", "imperatif-présent")},
        }



import pytest
from typing import cast

from verbecc.src.conjugator.conjugator import Conjugator, MoodsConjugation


@pytest.fixture(scope="module")
def cg():
    cg = Conjugator(lang="it")
    yield cg


def test_all_verbs_have_templates(cg):
    verbs = cg.get_verbs()
    template_names = cg.get_template_names()
    missing_templates = set()
    for verb in verbs:
        if verb.template not in template_names:
            missing_templates.add(verb.template)
    assert len(missing_templates) == 0


@pytest.mark.parametrize(
    "infinitive,mood,tense,expected_result",
    [
        (
            "avere",
            "indicativo",
            "presente",
            ["io ho", "tu hai", "lui ha", "noi abbiamo", "voi avete", "loro hanno"],
        ),
        (
            "avere",
            "indicativo",
            "imperfetto",
            [
                "io avevo",
                "tu avevi",
                "lui aveva",
                "noi avevamo",
                "voi avevate",
                "loro avevano",
            ],
        ),
        (
            "avere",
            "indicativo",
            "passato-remoto",
            [
                "io ebbi",
                "tu avesti",
                "lui ebbe",
                "noi avemmo",
                "voi aveste",
                "loro ebbero",
            ],
        ),
        (
            "avere",
            "indicativo",
            "futuro",
            [
                "io avrò",
                "tu avrai",
                "lui avrà",
                "noi avremo",
                "voi avrete",
                "loro avranno",
            ],
        ),
    ],
)
def test_inflector_it_conjugate_mood_tense(
    cg, infinitive, mood, tense, expected_result
):
    assert cg.conjugate_mood_tense(infinitive, mood, tense) == expected_result


def test_inflector_it_conjugate(cg):
    assert cg.conjugate("avere") != None


def test_inflector_it_add_subjunctive_relative_pronoun(cg):
    assert (
        cg._inflector._add_subjunctive_relative_pronoun("io abbia", "")
        == "che io abbia"
    )


@pytest.mark.parametrize(
    "person,gender,is_reflexive,expected_result",
    [
        ("1s", "m", False, "io"),
        ("1s", "m", True, "mi"),
        ("2s", "m", False, "tu"),
        ("2s", "m", True, "ti"),
        ("3s", "m", False, "lui"),
        ("3s", "m", True, "si"),
        ("3s", "f", False, "lei"),
        ("3s", "f", True, "si"),
        ("1p", "m", False, "noi"),
        ("1p", "m", True, "ci"),
        ("2p", "m", False, "voi"),
        ("2p", "m", True, "vi"),
        ("3p", "m", False, "loro"),
        ("3p", "m", True, "si"),
        ("3p", "f", False, "loro"),
        ("3p", "f", True, "si"),
    ],
)
def test_inflector_it_get_default_pronoun(
    cg, person, gender, is_reflexive, expected_result
):
    assert (
        cg._inflector._get_default_pronoun(person, gender, is_reflexive=is_reflexive)
        == expected_result
    )


@pytest.mark.parametrize(
    "infinitive,expected_result",
    [
        (
            "avere",
            ["io ho", "tu hai", "lui ha", "noi abbiamo", "voi avete", "loro hanno"],
        ),
        (
            "essere",
            ["io sono", "tu sei", "lui è", "noi siamo", "voi siete", "loro sono"],
        ),
        (
            "alzare",
            [
                "io alzo",
                "tu alzi",
                "lui alza",
                "noi alziamo",
                "voi alzate",
                "loro alzano",
            ],
        ),
    ],
)
def test_indicative_present(cg, infinitive, expected_result):
    conj = cg.conjugate(infinitive)
    moods_conj = cast(MoodsConjugation, conj["moods"])
    assert moods_conj["indicativo"]["presente"] == expected_result


@pytest.mark.parametrize(
    "infinitive,expected_result",
    [
        (
            "avere",
            [
                "io ho avuto",
                "tu hai avuto",
                "lui ha avuto",
                "noi abbiamo avuto",
                "voi avete avuto",
                "loro hanno avuto",
            ],
        ),
        (
            "essere",
            [
                "io sono stato",
                "tu sei stato",
                "lui è stato",
                "noi siamo stato",
                "voi siete stato",
                "loro sono stato",
            ],
        ),
        (
            "alzare",
            [
                "io ho alzato",
                "tu hai alzato",
                "lui ha alzato",
                "noi abbiamo alzato",
                "voi avete alzato",
                "loro hanno alzato",
            ],
        ),
    ],
)
def test_passato_prossimo(cg, infinitive, expected_result):
    conj = cg.conjugate(infinitive)
    moods_conj = cast(MoodsConjugation, conj["moods"])
    assert moods_conj["indicativo"]["passato-prossimo"] == expected_result


@pytest.mark.skip("TODO: fix")
@pytest.mark.parametrize(
    "infinitive,expected_result",
    [
        (
            "alzarsi",
            [
                "io mi alzo",
                "tu ti alzi",
                "lei si alza",
                "noi ci alziamo",
                "voi vi alzate",
                "loro si alzano",
            ],
        ),
    ],
)
def test_alzarsi_indicative_present(cg, infinitive, expected_result):
    conj = cg.conjugate(infinitive)
    moods_conj = cast(MoodsConjugation, conj["moods"])
    assert moods_conj["indicativo"]["presente"] == expected_result


@pytest.mark.skip("TODO: fix")
@pytest.mark.parametrize(
    "infinitive,expected_result",
    [
        (
            "alzarsi",
            [
                "io mi sono alzato",
                "tu ti sei alzato",
                "lei si è alzato",
                "noi ci siamo alzati",
                "voi vi siete alzati",
                "loro si sono alzate",
            ],
        ),
    ],
)
def test_alzarsi_passato_prossimo(cg, infinitive, expected_result):
    conj = cg.conjugate(infinitive)
    moods_conj = cast(MoodsConjugation, conj["moods"])
    assert moods_conj["indicativo"]["passato-prossimo"] == expected_result



from typing import Dict, List, Tuple

from verbecc.src.inflectors.inflector import Inflector
from verbecc.src.conjugator.conjugation_object import ConjugationObjects


class InflectorPt(Inflector):
    @property
    def lang(self) -> str:
        return "pt"

    def __init__(self):
        super(InflectorPt, self).__init__()

    def _add_subjunctive_relative_pronoun(self, s: str, tense_name: str):
        if tense_name == "presente":
            return "que " + s
        elif tense_name == "pretérito-imperfeito":
            return "se " + s
        elif tense_name == "futuro":
            return "quando " + s
        return s

    def _add_adverb_if_applicable(self, s: str, mood_name: str, tense_name: str):
        if mood_name == "imperativo" and tense_name == "negativo":
            return "não " + s
        elif mood_name == "infinitivo" and tense_name == "infinitivo-pessoal-presente":
            return "por " + s
        return s

    def _add_reflexive_pronoun_or_pronoun_suffix_if_applicable(
        self, s: str, is_reflexive: bool, mood_name: str, tense_name: str, person
    ):
        imperative = mood_name == "imperativo"
        if imperative or (
            mood_name == "infinitivo" and tense_name == "infinitivo-pessoal-presente"
        ):
            s += " " + self._get_pronoun_suffix(person, imperative=imperative)
        return s

    def _get_default_pronoun(self, person, gender="m", is_reflexive=False):
        ret = ""
        if person == "1s":
            ret = "eu"
            if is_reflexive:
                ret += " me"
        elif person == "2s":
            ret = "tu"
            if is_reflexive:
                ret += " te"
        elif person == "3s":
            ret = "ele"
            if gender == "f":
                ret = "ela"
            if is_reflexive:
                ret += " se"
        elif person == "1p":
            ret = "nós"
            if is_reflexive:
                ret += " nos"
        elif person == "2p":
            ret = "vós"
            if is_reflexive:
                ret += " se"
        elif person == "3p":
            ret = "eles"
            if gender == "f":
                ret = "elas"
            if is_reflexive:
                ret += " se"
        return ret

    def _get_pronoun_suffix(self, person, gender="m", imperative=True):
        ret = ""
        if person == "1s":
            ret = "eu"
        if person == "2s":
            ret = "tu"
        elif person == "3s":
            ret = "você"
            if not imperative:
                ret = "ele"
        elif person == "1p":
            ret = "nós"
        elif person == "2p":
            ret = "vós"
        elif person == "3p":
            ret = "vocês"
            if not imperative:
                ret = "eles"
        return ret

    def _get_tenses_conjugated_without_pronouns(self) -> List[str]:
        return [
            "particípio",
            "infinitivo",
            "infinitivo-pessoal-presente",
            "infinitivo-pessoal-composto",
            "afirmativo",
            "negativo",
            "gerúndio",
        ]

    def _get_auxilary_verb(
        self,
        co: ConjugationObjects,
        mood_name: str,
        tense_name: str,
    ) -> str:
        return "ter"

    def _get_infinitive_mood_name(self):
        return "infinitivo"

    def _get_indicative_mood_name(self):
        return "indicativo"

    def _get_subjunctive_mood_name(self) -> str:
        return "subjuntivo"

    def _get_conditional_mood_name(self):
        return "condicional"

    def _get_participle_mood_name(self) -> str:
        return "particípio"

    def _get_participle_tense_name(self) -> str:
        return "particípio"

    def _get_compound_conjugations_aux_verb_map(
        self,
    ) -> Dict[str, Dict[str, Tuple[str, ...]]]:
        return {
            "indicativo": {
                "pretérito-perfeito-composto": ("indicativo", "presente"),
                "pretérito-mais-que-perfeito-composto": (
                    "indicativo",
                    "pretérito-imperfeito",
                ),
                "pretérito-mais-que-perfeito-anterior": (
                    "indicativo",
                    "pretérito-mais-que-perfeito",
                ),
                "futuro-do-presente-composto": ("indicativo", "futuro-do-presente"),
            },
            "subjuntivo": {
                "pretérito-perfeito": ("subjuntivo", "presente"),
                "pretérito-mais-que-perfeito": ("subjuntivo", "pretérito-imperfeito"),
                "futuro-composto": ("subjuntivo", "futuro"),
            },
            "condicional": {
                "futuro-do-pretérito-composto": ("condicional", "futuro-do-pretérito")
            },
            "infinitivo": {
                "infinitivo-pessoal-composto": (
                    "infinitivo",
                    "infinitivo-pessoal-presente",
                )
            },
        }



from lxml import etree, objectify
import os

"""
This script was used to remove unwanted tense elements
(e.g. compound tenses) from the mlconjug conjugation XML files
"""

working_dir = "../verbecc/data"
in_file = "conjugations-ro.xml"
out_file = "conjugations-ro.mod.xml"


def remove_mood_tense(root, tenses_to_remove):
    removed_elem_cnt = 0
    for template_elem in root:
        if template_elem.tag == "template":
            for mood_elem in template_elem:
                for tense_elem in mood_elem:
                    if tense_elem.tag in tenses_to_remove:
                        mood_elem.remove(tense_elem)
                        removed_elem_cnt += 1
    print("removed {} elements".format(removed_elem_cnt))


def remove_mood(root, moods_to_remove):
    removed_elem_cnt = 0
    for template_elem in root:
        if template_elem.tag == "template":
            for mood_elem in template_elem:
                if mood_elem.tag in moods_to_remove:
                    template_elem.remove(mood_elem)
                    removed_elem_cnt += 1
    print("removed {} elements".format(removed_elem_cnt))


def move_tense(root, tense_name, old_mood, new_mood, remove_old_mood):
    moved_elem_cnt = 0
    for template_elem in root:
        if template_elem.tag == "template":
            tense_elem_to_move = None
            # find tense to move
            for mood_elem in template_elem:
                if mood_elem.tag == old_mood:
                    for tense_elem in mood_elem:
                        if tense_elem.tag == tense_name:
                            tense_elem_to_move = tense_elem
                            mood_elem.remove(tense_elem)
                            if remove_old_mood:
                                mood_elem.getparent().remove(mood_elem)
                            break
            # Now move it
            if tense_elem_to_move is not None:
                for mood_elem in template_elem:
                    if mood_elem.tag == new_mood:
                        mood_elem.append(tense_elem)
                        moved_elem_cnt += 1
    print("moved {} elements".format(moved_elem_cnt))


def main():
    parser = etree.XMLParser(dtd_validation=False, encoding="utf-8")
    tree = etree.parse(os.path.join(working_dir, in_file), parser)
    root = tree.getroot()

    remove_mood(root, ["Conditional"])
    # remove_mood_tense(root, ['perfect'])
    # move_tense(root, 'Viitor-II-popular', 'Viitor', 'Indicativ', True)

    with open(os.path.join(working_dir, out_file), "wb") as f:
        objectify.deannotate(root, cleanup_namespaces=True)
        xml = etree.tostring(
            tree,
            encoding="utf-8",
            method="xml",
            pretty_print=True,
            xml_declaration=True,
        )
        f.write(xml)


if __name__ == "__main__":
    main()



from typing import Dict, List, Tuple

from verbecc.src.inflectors.inflector import Inflector
from verbecc.src.conjugator.conjugation_object import ConjugationObjects


class InflectorEs(Inflector):
    @property
    def lang(self) -> str:
        return "es"

    def __init__(self):
        super(InflectorEs, self).__init__()

    def _add_adverb_if_applicable(self, s: str, mood_name: str, tense_name: str) -> str:
        if mood_name == "imperativo" and tense_name == "negativo":
            return "no " + s
        return s

    def _get_default_pronoun(
        self, person: str, gender: str = "m", is_reflexive: bool = False
    ) -> str:
        ret = ""
        if person == "1s":
            ret = "yo"
            if is_reflexive:
                ret += " me"
        elif person == "2s":
            ret = "tú"
            if is_reflexive:
                ret += " te"
        elif person == "3s":
            ret = "él"
            if gender == "f":
                ret = "ella"
            if is_reflexive:
                ret += " se"
        elif person == "1p":
            ret = "nosotros"
            if is_reflexive:
                ret += " nos"
        elif person == "2p":
            ret = "vosotros"
            if is_reflexive:
                ret += " os"
        elif person == "3p":
            ret = "ellos"
            if gender == "f":
                ret = "ellas"
            if is_reflexive:
                ret += " se"
        return ret

    def _get_tenses_conjugated_without_pronouns(self) -> List[str]:
        return ["participo", "gerundio", "infinitivo", "afirmativo", "negativo"]

    def _get_auxilary_verb(
        self,
        co: ConjugationObjects,
        mood_name: str,
        tense_name: str,
    ) -> str:
        return "haber"

    def _get_infinitive_mood_name(self):
        return "infinitivo"

    def _get_indicative_mood_name(self):
        return "indicativo"

    def _get_subjunctive_mood_name(self):
        return "subjuntivo"

    def _get_conditional_mood_name(self):
        return "condicional"

    def _get_participle_mood_name(self) -> str:
        return "participo"

    def _get_participle_tense_name(self) -> str:
        return "participo"

    def _get_alternate_hv_inflection(self, s: str) -> str:
        if s.endswith("hay"):
            return s[:-1]
        return s

    def _get_compound_conjugations_aux_verb_map(
        self,
    ) -> Dict[str, Dict[str, Tuple[str, ...]]]:
        return {
            "indicativo": {
                "pretérito-perfecto-compuesto": ("indicativo", "presente"),
                "pretérito-pluscuamperfecto": ("indicativo", "pretérito-imperfecto"),
                "pretérito-anterior": ("indicativo", "pretérito-perfecto-simple"),
                "futuro-perfecto": ("indicativo", "futuro"),
            },
            "condicional": {"perfecto": ("condicional", "presente")},
            "subjuntivo": {
                "pretérito-perfecto": ("subjuntivo", "presente"),
                "pretérito-pluscuamperfecto-1": (
                    "subjuntivo",
                    "pretérito-imperfecto-1",
                ),
                "pretérito-pluscuamperfecto-2": (
                    "subjuntivo",
                    "pretérito-imperfecto-2",
                ),
                "futuro-perfecto": ("subjuntivo", "futuro"),
            },
        }



from typing import Dict, Tuple

SUPPORTED_LANGUAGES: Dict[str, str] = {
    "ca": "català",
    "es": "español",
    "fr": "français",
    "it": "italiano",
    "pt": "português",
    "ro": "română",
}

# This refers to grammatical person ('usted' is 3s despite being semantically 2p)
PERSONS: Tuple[str, str, str, str, str, str] = ("1s", "2s", "3s", "1p", "2p", "3p")

IMPERATIVE_PRESENT_PERSONS: Tuple[str, str, str] = ("2s", "1p", "2p")

PARTICIPLE_INFLECTIONS: Tuple[str, str, str, str] = ("ms", "mp", "fs", "fp")

ALPHABET = {
    "fr": {"vowels": "aáàâeêéèiîïoôöœuûùy", "consonants": "bcçdfghjklmnpqrstvwxyz"},
    "en": {"vowels": "aeiouy", "consonants": "bcdfghjklmnpqrstvwxyz"},
    "ca": {"vowels": "aáàâeéèiïoôuûùy", "consonants": "bcdfghjklmnñpqrstvwxyz"},
    "es": {"vowels": "aáeiíoóuúy", "consonants": "bcdfghjklmnñpqrstvwxyz"},
    "it": {"vowels": "aàeéèiìîoóòuùy", "consonants": "bcdfghjklmnpqrstvwxyz"},
    "pt": {"vowels": "aàãááeêéiíoóõuúy", "consonants": "bcçdfghjklmnpqrstvwxyz"},
    "ro": {"vowels": "aăâeiîouy", "consonants": "bcdfghjklmnpqrsșştțţvwxyz"},
}



from typing import Dict, Tuple

from verbecc.src.inflectors.inflector import Inflector
from verbecc.src.utils import string_utils

VERBS_CONJUGATED_WITH_ESSERE = [
    "essere",
    "andare",
    "arrivare",
    "cadere",
    "entrare",
    "partire",
    "rimanere",
    "uscire",
    "venire",
    "stare",
    "passare",
    "diventare",
    "crescere",
    "morire",
    "nascere",
]


class InflectorIt(Inflector):
    @property
    def lang(self) -> str:
        return "it"

    def __init__(self):
        super(InflectorIt, self).__init__()

    def _split_reflexive(self, infinitive) -> Tuple[bool, str]:
        is_reflexive = False
        if infinitive.startswith("si "):
            is_reflexive = True
            infinitive = infinitive[3:]
        elif infinitive.startswith("s'"):
            is_reflexive = True
            infinitive = infinitive[2:]
        return is_reflexive, infinitive

    def _add_reflexive_pronoun(self, s):
        if string_utils.starts_with_vowel(s, h_is_vowel=True):
            return "s'" + s
        else:
            return "si " + s

    def _add_subjunctive_relative_pronoun(self, s, tense_name):
        return "che " + s

    def _get_default_pronoun(self, person, gender="m", is_reflexive=False):
        ret = ""
        if person == "1s":
            ret = "io"
            if is_reflexive:
                ret = "mi"
        elif person == "2s":
            ret = "tu"
            if is_reflexive:
                ret = "ti"
        elif person == "3s":
            ret = "lui"
            if gender == "f":
                ret = "lei"
            if is_reflexive:
                ret = "si"
        elif person == "1p":
            ret = "noi"
            if is_reflexive:
                ret = "ci"
        elif person == "2p":
            ret = "voi"
            if is_reflexive:
                ret = "vi"
        elif person == "3p":
            ret = "loro"
            if is_reflexive:
                ret = "si"
        return ret

    def _get_tenses_conjugated_without_pronouns(self):
        return [
            "affermativo",
            "negativo",
            "Negativo",
            "gerundio",
            "participio-presente",
            "participio-passato",
        ]

    def _get_auxilary_verb(self, co, mood_name, tense_name):
        ret = "avere"
        if co.verb.infinitive in VERBS_CONJUGATED_WITH_ESSERE or co.is_reflexive:
            ret = "essere"
        return ret

    def _get_infinitive_mood_name(self):
        return "infinitivo"

    def _get_indicative_mood_name(self):
        return "indicativo"

    def _get_subjunctive_mood_name(self):
        return "congiuntivo"

    def _get_conditional_mood_name(self):
        return "condizionale"

    def _get_participle_mood_name(self):
        return "participio"

    def _get_participle_tense_name(self):
        return "participio-passato"

    def _get_compound_conjugations_aux_verb_map(
        self,
    ) -> Dict[str, Dict[str, Tuple[str, ...]]]:
        return {
            "indicativo": {
                "passato-prossimo": ("indicativo", "presente"),
                "trapassato-prossimo": ("indicativo", "imperfetto"),
                "trapassato-remoto": ("indicativo", "passato-remoto"),
                "futuro-anteriore": ("indicativo", "futuro"),
            },
            "congiuntivo": {
                "passato": ("congiuntivo", "presente"),
                "trapassato": ("congiuntivo", "imperfetto"),
            },
            "condizionale": {"passato": ("condizionale", "presente")},
        }
